@@ -1,19 +1,16 @@
 use super::types::{
-    subscribe::{SubscribeRequest, SubscriptionEvent}, Error as APIError, RequestId, Response, Result as APIResult,
+    subscribe::{SubscribeRequest, SubscriptionEvent},
+    RequestId, WebsocketResponse, JsonRPCResponse, ErrorResponse, JsonRPCResponseResult,
 };
-use super::Error;
 use async_trait::async_trait;
 use futures::{
-    channel::{mpsc, mpsc::UnboundedReceiver, oneshot},
-    stream::Map,
-    task::Context,
+    channel::mpsc,
     SinkExt, Stream, StreamExt,
 };
 use reqwest::{header::CONTENT_TYPE, Client};
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 use serde_json::{json, Value};
 use std::collections::HashMap;
-use std::convert::TryInto;
 use std::fmt::Debug;
 use std::pin::Pin;
 use std::sync::atomic::{AtomicU64, Ordering};
@@ -24,16 +21,16 @@ use tokio_tungstenite::{
 };
 use url::{ParseError, Url};
 
-#[async_trait(?Send)]
+#[async_trait]
 pub trait Transport {
-    async fn send_request<Params: Serialize, Res: DeserializeOwned + Debug>(
+    async fn send_request<Params: Serialize + Send, Res: DeserializeOwned + Debug + Send>(
         &self,
         method: &str,
         params: Params,
     ) -> Result<Res, TransportError>;
 }
 
-#[async_trait(?Send)]
+#[async_trait]
 pub trait DuplexTransport: Transport {
     async fn subscribe<T: DeserializeOwned>(
         &self,
@@ -48,9 +45,10 @@ pub enum TransportError {
     Error(&'static str),
     InvalidEndpoint(ParseError),
     ReqwestError(reqwest::Error),
+    JSONError(serde_json::Error),
     WSError(WSError),
     ErrorResponse(String),
-    APIError(Value),
+    APIError(ErrorResponse),
 }
 
 impl From<reqwest::Error> for TransportError {
@@ -66,21 +64,21 @@ impl From<WSError> for TransportError {
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub struct JsonRPCRequest<T: Serialize> {
+pub struct JsonRPCRequest<T: Serialize + Send> {
     pub id: RequestId,
     pub method: String,
     pub params: T,
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub struct WebSocketRPCRequest<T: Serialize> {
+pub struct WebSocketRPCRequest<T: Serialize + Send> {
     pub id: RequestId,
     pub command: String,
     #[serde(flatten)]
     pub params: T,
 }
 
-unsafe impl<T: Serialize> Send for JsonRPCRequest<T> {}
+unsafe impl<T: Serialize + Send> Send for JsonRPCRequest<T> {}
 
 pub struct HTTP {
     counter: AtomicU64,
@@ -94,31 +92,30 @@ impl HTTP {
     }
 }
 
-#[async_trait(?Send)]
+#[async_trait]
 impl Transport for HTTP {
-    async fn send_request<Params: Serialize, Res: DeserializeOwned + Debug>(
+    async fn send_request<Params: Serialize + Send, Res: DeserializeOwned + Debug + Send>(
         &self,
         method: &str,
         params: Params,
     ) -> Result<Res, TransportError> {
-        match self
-            .inner
+        let json_str = serde_json::to_string(&JsonRPCRequest {
+            id: self.counter.fetch_add(1u64, Ordering::SeqCst),
+            method: method.to_owned(),
+            params: vec![params],
+        })
+        .map_err(|e| TransportError::JSONError(e))?;
+        let client = self.inner.clone();
+        let res = client
             .post(self.base_url.clone())
             .header(CONTENT_TYPE, "application/json")
-            .json(&JsonRPCRequest {
-                id: self.counter.fetch_add(1u64, Ordering::SeqCst),
-                method: method.to_owned(),
-                params: vec![params],
-            })
+            .body(json_str)
             .send()
-            .await?
-            .json::<Response<Res>>()
-            .await
-            .map_err(|e| TransportError::ReqwestError(e))?
-            .result
-        {
-            APIResult::Ok(result) => Ok(result),
-            APIResult::Error(e) => Err(TransportError::APIError(e)),
+            .await?;
+        let json = res.json::<JsonRPCResponse<Res>>().await;
+        match json.map_err(|e| TransportError::ReqwestError(e))?.result {
+            JsonRPCResponseResult::Success(success) => Ok(success.result),
+            JsonRPCResponseResult::Error(e) => Err(TransportError::APIError(e)),
         }
     }
 }
@@ -149,12 +146,12 @@ pub enum PendingRequest {
     Call {
         id: RequestId,
         request: WebSocketRPCRequest<Value>,
-        response: mpsc::Sender<Response<Value>>,
+        response: mpsc::Sender<WebsocketResponse<Value>>,
     },
     Subscription {
         id: RequestId,
         request: WebSocketRPCRequest<Value>,
-        channel: mpsc::UnboundedSender<Response<Value>>,
+        channel: mpsc::UnboundedSender<WebsocketResponse<Value>>,
     },
 }
 
@@ -177,9 +174,9 @@ impl WebSocket {
     }
 }
 
-#[async_trait(?Send)]
+#[async_trait]
 impl Transport for WebSocket {
-    async fn send_request<Params: Serialize, Res: DeserializeOwned + Debug>(
+    async fn send_request<Params: Serialize + Send, Res: DeserializeOwned + Debug + Send>(
         &self,
         method: &str,
         params: Params,
@@ -203,21 +200,24 @@ impl Transport for WebSocket {
             .send(request)
             .await
             .map_err(|e| TransportError::ErrorResponse(format!("sending: {:?}", e)))?; //TODO: Add error type for websocket send error
-        let response: Response<Value> = r
+        let response: WebsocketResponse<Value> = r
             .take(1)
-            .collect::<Vec<Response<Value>>>()
+            .collect::<Vec<WebsocketResponse<Value>>>()
             .await
             .first()
             .unwrap()
             .clone();
-        match response.result {
-            APIResult::Ok(result) => Ok(serde_json::from_value(result).unwrap()),
-            APIResult::Error(e) => Err(TransportError::APIError(e)),
+        match response {
+            WebsocketResponse::Success(success) => {
+                println!("{:?}", success);
+                Ok(serde_json::from_value(success.result).unwrap())
+            },
+            WebsocketResponse::Error(e) => Err(TransportError::APIError(e)),
         }
     }
 }
 
-#[async_trait(?Send)]
+#[async_trait]
 impl DuplexTransport for WebSocket {
     async fn subscribe<T: DeserializeOwned>(
         &self,
@@ -242,9 +242,9 @@ impl DuplexTransport for WebSocket {
             .send(req)
             .await
             .map_err(|e| TransportError::ErrorResponse(format!("sending: {:?}", e)))?; //TODO: Add error type for websocket send error
-        let stream = r.map(|response| match response.result {
-            APIResult::Ok(result) => Ok(serde_json::from_value(result).unwrap()),
-            APIResult::Error(e) => Err(TransportError::APIError(e)),
+        let stream = r.map(|response| match response {
+            WebsocketResponse::Success(success) => Ok(serde_json::from_value(success.result).unwrap()),
+            WebsocketResponse::Error(e) => Err(TransportError::APIError(e)),
         });
         Ok(Box::pin(stream))
     }
@@ -274,13 +274,12 @@ impl WebSocketBuilder {
         tokio::spawn(async move {
             read.for_each(|message| async {
                 let data = message.unwrap().into_data();
-                let s = String::from_utf8_lossy(&data);
-                let res: Option<Response<Value>> = serde_json::from_slice(&data).ok();
+                let res: Option<WebsocketResponse<Value>> = serde_json::from_slice(&data).ok();
                 match res {
                     Some(res) => {
                         let pr = pending_requests
                             .lock()
-                            .map(|p| p.get(&res.id.unwrap()).unwrap().clone())
+                            .map(|p| p.get(&res.get_id().unwrap()).unwrap().clone())
                             .unwrap();
                         match pr {
                             PendingRequest::Call { response, .. } => {
